import { app } from '@/app.ts';
import { FileSyncModel } from '@/file-sync/model.ts';
import { logger } from '@/modules/logger.ts';
import { oss, SyncFile, SyncType } from '@/modules/minio.ts';
import { BucketItemStat } from 'minio';
const sleep = (ms: number) => new Promise((resolve) => setTimeout(resolve, ms));
type Result = {
  code: number;
  message?: string;
  data?: any;
  duration?: number;
  error?: string;
};

app
  .route({
    path: 'file-listener',
    key: 'db-check',
    description: '检查数据库中的文件, 获取minio的文件，如果不一致则更新',
  })
  .define(async (ctx) => {
    const where = {
      $or: [
        {
          checkedAt: {
            $lt: new Date(Date.now() - 1000 * 60 * 60 * 24 * 30), // 检查30天前未检查的文件
          },
        },
        {
          checkedAt: null, // 也检查从未检查过的文件
        },
      ],
    };
    const count = await FileSyncModel.count({ where });

    for (let i = 0; i < count; i += 10000) {
      // 每次处理10000个文件，避免过载
      // 分页查询所有的数据库，获取对应的name
      const dbFiles = await FileSyncModel.findAll({
        where,
        attributes: ['name'],
        raw: true,
        limit: 10000,
        offset: i,
      });
      if (!dbFiles || dbFiles.length === 0) {
        logger.info('No files to check in the database');
        continue;
      }
      for (const file of dbFiles) {
        const name = file.name;
        if (!name) continue;
        // 调用同步文件的任务
        app.call({
          path: 'file-listener',
          key: 'sync-file-task',
          payload: {
            data: {
              type: 'sync',
              name: name,
            },
          },
        });
      }
      sleep(100);
    }
  })
  .addTo(app);

app
  .route({
    path: 'file-listener',
    key: 'sync-all',
    description: '同步minio的所有文件到数据库',
  })
  .define(async (ctx) => {
    const startTime = Date.now();
    const runSync = async () => {
      return new Promise<Result>((resolve, reject) => {
        const dataStream = oss.client.listObjectsV2(oss.bucketName, '', true);
        dataStream.on('data', async (data) => {
          if (data.name?.startsWith?.('private')) return;
          app.call({
            path: 'file-listener',
            key: 'sync-file-task',
            payload: {
              data: data,
            },
          });
        });
        dataStream.on('end', () => {
          const endTime = Date.now();
          logger.info(`Sync completed in ${endTime - startTime} ms`);
          resolve({ code: 200, message: 'Sync completed successfully', duration: endTime - startTime });
        });
        dataStream.on('error', (err) => {
          console.error('Error during sync:', err);
          ctx.body = { message: 'Sync failed', error: err.message };
          resolve({ code: 500, message: 'Sync failed', error: err.message });
        });
      });
    };
    const res = await runSync();
    if (res.code === 200) {
      ctx.body = res.data;
    }
    ctx.throw(res.code, res.message || 'Sync operation completed');
  })
  .addTo(app);

app
  .route({
    path: 'file-listener',
    key: 'sync-file-task',
  })
  .define(async (ctx) => {
    const query = ctx.query;
    console.log('queue', query);
    const result = await app.call({
      path: 'file-listener',
      key: 'sync-file',
      payload: query,
    });
    if (result.code !== 200) {
      ctx.throw(result.code, result.message || 'Sync file task failed');
    }
    ctx.body = result.data;
  })
  .addTo(app);
app
  .route({
    path: 'file-listener',
    key: 'sync-file',
  })
  .define(async (ctx) => {
    const { data } = ctx.query;
    if (!data.name && !data.key) {
      ctx.throw(400, 'No data provided for sync');
    }
    const type: SyncType = data.type || 'sync';
    const name = data.name || data.key;
    logger.info('Syncing file:', name, 'Type:', type);
    let fileSync: FileSyncModel = null;
    if (type === 'delete') {
      const deleteNum = await FileSyncModel.destroy({
        where: { name },
      });
      if (deleteNum) {
        logger.info('File deleted from sync model:', name);
      }
      return;
    }
    const bucketItemStat: BucketItemStat = await oss.statObject(name);

    if (type === 'create' || type === 'update' || type === 'sync') {
      const [fileSync, created] = await FileSyncModel.findOrCreate({
        where: { name },
        defaults: {
          name,
          hash: bucketItemStat.etag || '',
          stat: bucketItemStat || {},
          data: {},
          checkedAt: new Date(),
        },
      });
      if (created) {
        logger.info('File created in sync model:', name);
        return;
      }
      const result = await fileSync.updateStat(bucketItemStat);
      logger.info('File updated in sync model:', name, result);
      ctx.body = result;
      return;
    }

    logger.info('Processing file:', bucketItemStat);
  })
  .addTo(app);
